﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_90_degree_arcs
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private List<PointF> Points = new List<PointF>();

        private void picCanvas_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.Clear(picCanvas.BackColor);

            // Find the arcs.
            if (Points.Count == 2)
            {
                // Find the arcs' bounding rectangles.
                RectangleF rect1, rect2;
                float start_angle1, start_angle2, sweep_angle1, sweep_angle2;
                FindArcs(Points[0], Points[1], out rect1, out rect2,
                    out start_angle1, out start_angle2,
                    out sweep_angle1, out sweep_angle2);

                // Draw the bounding rectangles.
                using (Pen pen = new Pen(Color.Red, 0))
                {
                    // Draw the bounding rectangles.
                    pen.DashPattern = new float[] { 1, 2 };
                    e.Graphics.DrawRectangle(pen, rect1);
                    pen.Color = Color.Green;
                    e.Graphics.DrawRectangle(pen, rect2);

                    // Draw the bounding ellipses.
                    pen.Color = Color.Red;
                    pen.DashPattern = new float[] { 5, 5 };
                    e.Graphics.DrawEllipse(pen, rect1);
                    pen.Color = Color.Green;
                    e.Graphics.DrawEllipse(pen, rect2);
                }

                // Draw the arcs.
                using (Pen pen = new Pen(Color.Red, 2))
                {
                    e.Graphics.DrawArc(pen, rect1, start_angle1, sweep_angle1);
                    pen.Color = Color.Green;
                    e.Graphics.DrawArc(pen, rect2, start_angle2, sweep_angle2);
                }
            }

            // Draw the points.
            foreach (PointF point in Points)
                DrawPoint(e.Graphics, Brushes.Green, point);
        }

        private void picCanvas_MouseClick(object sender, MouseEventArgs e)
        {
            if (Points.Count == 2) Points = new List<PointF>();
            Points.Add(e.Location);
            picCanvas.Refresh();
        }

        private void DrawPoint(Graphics gr, Brush brush, PointF point)
        {
            const float radius = 3;
            RectangleF rect = new RectangleF(
                point.X - radius,
                point.Y - radius,
                2 * radius, 2 * radius);
            gr.FillEllipse(brush, rect);
        }

        // Find 90 degree arcs connecting the two points.
        private void FindArcs(PointF point1, PointF point2,
            out RectangleF rect1, out RectangleF rect2,
            out float start_angle1, out float start_angle2,
            out float sweep_angle1, out float sweep_angle2)
        {
            // Find the rectangles' dimensions.
            float x_radius = Math.Abs(point1.X - point2.X);
            float y_radius = Math.Abs(point1.Y - point2.Y);

            // If the points have the same X or Y coordinate,
            // use midpoint circular arcs.
            if ((x_radius == 0) || (y_radius == 0))
            {
                FindMidpointArcs(point1, point2,
                    out rect1, out rect2,
                    out start_angle1, out start_angle2,
                    out sweep_angle1, out sweep_angle2);
                return;
            }

            // Make point1 be on the left.
            if (point2.X < point1.X)
            {
                // Swap them.
                PointF temp = point1;
                point1 = point2;
                point2 = temp;
            }

            // See whether point1 is above or below point2.
            if (point1.Y < point2.Y)
            {
                // point1 is above point2.
                rect1 = new RectangleF(
                    point1.X - x_radius,
                    point1.Y,
                    2 * x_radius,
                    2 * y_radius);
                rect2 = new RectangleF(
                    point1.X,
                    point1.Y - y_radius,
                    2 * x_radius,
                    2 * y_radius);
                start_angle1 = -90;
                start_angle2 = 90;
            }
            else
            {
                // point1 is below point2.
                rect1 = new RectangleF(
                    point1.X - x_radius,
                    point1.Y - 2 * y_radius,
                    2 * x_radius,
                    2 * y_radius);
                rect2 = new RectangleF(
                    point1.X,
                    point1.Y - y_radius,
                    2 * x_radius,
                    2 * y_radius);
                start_angle1 = 0;
                start_angle2 = 180;
            }
            sweep_angle1 = 90;
            sweep_angle2 = 90;
        }

        // Find the arcs connecting two points
        private void FindMidpointArcs(PointF point0, PointF point1,
            out RectangleF rect1, out RectangleF rect2,
            out float start_angle1, out float start_angle2,
            out float sweep_angle1, out float sweep_angle2)
        {
            // Find the midpoint.
            float cx = (point0.X + point1.X) / 2f;
            float cy = (point0.Y + point1.Y) / 2f;

            // Find the radius.
            float radius = (float)(Distance(point0, point1) / 2.0);

            // Create the rectangles.
            rect1 = new RectangleF(
                cx - radius, cy - radius,
                2 * radius, 2 * radius);
            rect2 = rect1;

            // Find the start angles.
            start_angle1 = (float)(180 / Math.PI * Math.Atan2(
                point1.Y - point0.Y,
                point1.X - point0.X));
            start_angle2 = (start_angle1 + 180) % 360;
            sweep_angle1 = 180;
            sweep_angle2 = 180;
        }

        private double Distance(PointF point1, PointF point2)
        {
            float dx = point1.X - point2.X;
            float dy = point1.Y - point2.Y;
            return Math.Sqrt(dx * dx + dy * dy);
        }
    }
}
